<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://AmorStyle
 * @since      1.0.0
 *
 * @package    tweb3mmexplore
 * @subpackage tweb3mmexplore/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    tweb3mmexplore
 * @subpackage tweb3mmexplore/public
 * @author     AmorStyle <AmorStyleYoga@Hotmail.com>
 */
class tweb3mmexplore_shortcode 
{
	// Holds the shortcode options
	private $Localatts = array();

	// boolean - ether true of false. false by default.
	private $debug_spew;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct(  )  // $plugin_name, $version
	{
		$this->debug_spew = false;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in tweb3mmexplore_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The tweb3mmexplore_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/tweb3mmexplore-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in tweb3mmexplore_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The tweb3mmexplore_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/tweb3mmexplore-public.js', array( 'jquery' ), $this->version, false );

		// for test code
		wp_enqueue_script( $this->plugin_name."test", plugin_dir_url( __FILE__ ) . 'js/tweb3mmexplore-public-test.js', array( 'jquery' ), $this->version, false );
	}


	// This routine reads the shortcode and sets member variables 
	private function parse_options($atts)
	{
		// Setup the default here
		$defaults = array(	'item' => 'connect',
							'html' => 'true',
							'size' => 'small',
							'debug' => 'false');
							
		$this->Localatts = shortcode_atts( $defaults, $atts, 'connect' );
	}
	private function DEBUG()
	{
		// output what's true or set
		echo "item: ".$this->Localatts['item']."<br/>";
		echo "html: ".$this->Localatts['html']."<br/>";
		echo "size: ".$this->Localatts['size']."<br/>";
		echo "debug: ".$this->Localatts['debug']."<br/>";
		//echo "decimals: ".$this->Localatts['decimals']."<br/>";
	}

	private function connectButtonHTML()
	{
		// if the user wants debug strings in the console.
		if($this->Localatts['debug'] == 'true' )
		{
			?>
			<html>
			<p hidden>Debug: <span id="turnondebug"></span></p>
			</html>
			<?php
		}

		// Normal settings path
		if($this->Localatts['size'] == 'small' )
		{
			?>
			<html>
			<div >
			<div><span id="cbStatusId"></span></div>
			<button id="connectButtonId" disabled>javascript not running</button>
			</div>
			</html>
			<?php
		}
//
//			<div><span id="cbStatusId"></span></div>
//			<p hidden>Account: <span id="accountId"></span></p>
//			<button id="connectButtonId" disabled>javascript not running</button>
//		    <p>ChainId: <span id="chainId"></span></p>
//			<p hidden>Balance: <span id="balanceId"></span></p>
//			<p hidden>BlockHeight: <span id="blockId"></span></p>
//
		else
		{
			// size is full
			?>
			<html>
			<div style="border:1px solid black;">
			<div style="margin-top: 2%;margin-bottom: 2%;margin-right:2%;margin-left:2%">
			<div>
			<p>ChainId: <span id="chainId"></span><br>
				Account: <span id="accountId"></span><br>
				BlockHeight: <span id="blockId"></span><br>
				Balance: <span id="balanceId"></span></p>

			</div>
			<div><span id="cbStatusId"></span></div>
			<button id="connectButtonId" disabled>Empty text here</button>
			</div>
			</div>
			</html>
			<?php
		}
//			<p>ChainId: <span id="chainId"></span><br>
//				Account: <span id="accountId"></span><br>
//				BlockHeight: <span id="blockId"></span><br>
//				Balance: <span id="balanceId"></span></p>

	}

	private function signButtonHTML()
	{
		$theNonce = wp_create_nonce('my-nonce');
		// if the user wants debug strings in the console.
		if($this->Localatts['debug'] == 'true' )
		{
			?>
			<html>
			<p hidden>Debug: <span id="turnondebug"></span></p>
			</html>
			<?php
		}

		// Normal settings path
		?>
		<html>
		<div >
		<div style="border:1px solid black;">
		<div style="margin-top: 2%;margin-bottom: 2%;margin-right:2%;margin-left:2%">
		<p>Sign Typed Data V4</p>
		
		<div hidden><span id="signTypedDataV4MsgId">Describe what's being signed here</span></div>
		<div hidden><span id="signTypedDataV4SecretId"><?php echo $theNonce ?></span></div>
		<button id="signTypedData4Id" disabled>Sign</button>
		<div><span id="signTypedDataV4StatusId"></span></div>
		<p hidden>Result: <span id="signTypedDataV4ResultId"></span></p>

		</div>
		</div>
		</div>
		</html>
		<?php
	}
/*
		<div hidden><span id="signTypedDataV4MsgId">Describe what's being signed here</span></div>
		<div hidden><span id="signTypedDataV4SecretId"><?php echo $theNonce ?></span></div>
*/

//		<div><span id="signTypedDataV4StatusId"></span></div>
//		<button id="signTypedData4Id" disabled>Sign - not active</button>
//		<p>Result: <span id="signTypedDataV4ResultId"></span></p>
//
//		<button id="signTypedDataV4VerifyId" disabled >Verify - not active</button>
//      <p>Recovery result:<span id="signTypedDataV4VerifyResultId"></span></p>
//		<center>
//		</center>

	/*

	'item' - type of functionality to show. 
		'connectButton' - want the website to connect to MetaMask

	'size' - either small or full. small gives just the button for connectButton

	'html' - either true of false. Show the html as displayed with the plugin.
	*/
	public function main($atts)
	{
		// All information needs to be returned in a string.
		$output = ""; //"START SHORTCODE<br>";

		self::parse_options($atts, $output);
		//self::DEBUG();

		// Now call function to do work.
		$item = $this->Localatts['item'];
		if( $item == "connectButton")
		{
			if($this->Localatts['html'] == 'true' )
			{
				self::connectButtonHTML();
			}
		}
		else if( $item == "signButton")
		{
			if($this->Localatts['html'] == 'true' )
			{
				self::signButtonHTML();
			}
		}

		return $output; //."<br>END SHORTCODE<br>";	}
	}
}


//
// Main shortcode entrypoint. 
//
function tweb3mmexplore_shortcode( $atts )
{
	// ob_start() and ob_get_clean() are used so echo can be used in development
	ob_start();
	$Shortcode=new tweb3mmexplore_shortcode();
	$Text=$Shortcode->main($atts);

	// Need to determine what platform we're on
	$Location = site_url();
	if( stristr($Location,"localhost") != false )
	{
		// Debug spew here..
		return ob_get_clean().$Text;
	}
	else
	{
		// No Debug Spew.
		return ob_get_clean().$Text;
	}
}
