(function( $ ) {
	'use strict';

	/**
	 * All of the code for your public-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */

})(jQuery);



// files that do not touch the page
import CWeiNumber from './modules/cweinumber.js';
//import CTestWei from './tests/wei-tests.js'; // tests the cweinumber class

// files that have requirements...
import * as dom from './modules/domutils.js';
import * as cbtn from './modules/cconnectbutton.js';
//
// This signing functionality depends upon the connectionButton being present so it can provide
// the linked account number.
//
import * as sbtn from './modules/csignbutton.js';

const metaMaskURL = "https://metamask.io/"

// now create the class that handles the state changes for the connection button.
let cb = new cbtn.cconnectbutton()
let sb = new sbtn.csignButtonV4()

// The following code can be turned on by the debug option in the shortcode
const turnondebugStr = "turnondebug"
let turnondebugDiv
let turnOnDebug = false;

function DebugSpew(Str) {
    if (true == turnOnDebug) {
        console.log(Str)
        cb.turnOnDebug(true)
        if (sb.exists()) {
            sb.turnOnDebug(true)
        }
    }
}



//
// This function gets the balance for the connected account
//
async function theBalance() {

    let act = cb.getAccount()
    DebugSpew("Checking Balance of: " + act)

    // now get the balance...
    try {
        let bal = await ethereum.request({
            method: 'eth_getBalance',
            params: [
                act,
                'latest'
            ],
        })

        // Now set the balance item.
        let theWeiNum = new CWeiNumber(bal)
        let balFull = theWeiNum.full(true, 2, true)
        DebugSpew("full formated: " + balFull)

        cb.setBalance(balFull)

    } catch (err) {
        console.error(err)
    }
    DebugSpew("Checking Balance of: " + act + " Leaving")
}

async function theLatestBlockNumber() {

    // now get the latest block number...
    try {
        let blk = await ethereum.request({
            method: 'eth_blockNumber',
        })

        let dec = parseInt(blk,16)
        DebugSpew("latest block Number: " + dec + " (" + blk + ")")

        cb.setBlock(blk)

    } catch (err) {
        console.error(err)
    }
}

function queryNetworkForData() {
    if (cb.ready()) {
        theBalance()
        theLatestBlockNumber()
    }
    else {
        cb.setBalance(0)
        cb.setBlock(0)
    }
}

async function getNetworkAndChainId() {
    DebugSpew("getNetworkAndChainId() enter")
    try {
        const chainId = await ethereum.request({
            method: 'eth_chainId',
        })
        
        cb.handleChainIdChange(chainId)
        if (sb.exists()) {
            sb.handleChainIdChange(chainId)
        }
    } catch (err) {
        console.error(err)
    }
    DebugSpew("getNetworkAndChainId() leave")
}

/*
These two routines handle notifications from MetaMask. If the user changes the
network or wallet that is connected, this website will be notified so it can update.
*/
function handleNewChain(chainId) {
    DebugSpew("Notification handler handleNewChain enter")

    cb.handleChainIdChange(chainId)
    queryNetworkForData()

    if (sb.exists()) {
        sb.handleChainIdChange(chainId)
    }
    DebugSpew("Notification handler handleNewChain leave")
}
function handleNewAccounts(newAccounts) {
    DebugSpew("Notification handler handleNewAccounts enter")
    cb.handleAccountChange(newAccounts)

    queryNetworkForData()
    cb.updateState()

    if (sb.exists()) {
        sb.handleAccountChange(newAccounts)
    }
    DebugSpew("Notification handler handleNewAccounts leave")
}

/*
There are two possible opperations that need to be handled when the user clicks the button.

if the button is set to connect then
  send a eth_requestAccounts message which launches MetaMask so the user can connect
  if success then
    the call returns the array of connected wallets
  else
    fall back to previous wait state
else
  the button is set to 'learn how to install metamask' which launches website in tab

*/
const onClickHandler = async () => {

    DebugSpew("onClickHandler Enter")
    if (cb.exists()) {
        try {

            // Need to see what state the button was in inorder to determine the work that
            // needs to be done.

            if (cb.isConnect()) {
                //DebugSpew("NotLinked, requesting accounts...")

                const newAccounts = await ethereum.request({
                    method: 'eth_requestAccounts',
                })

                cb.handleAccountChange(newAccounts)
                queryNetworkForData()

            }
            else if (cb.isInstall) {

                DebugSpew("NotAvailable - install")
                // Send the user to the MetaMask website.
                window.open(metaMaskURL, "_blank")
            }
            else {
                console.warn("Invalid state")
            }

        } catch (error) {
            if (error.code == 4001) {
                DebugSpew(error.message)
                // user bailed
            }
            else {
                console.error(error)
            }
        }
        cb.updateState()
    }
    else {
        DebugSpew("no Button to setup")

    }
    DebugSpew("onClickHandler exit")
}

function onClickSetup() {
    let connectButton = cb.getButton()
    if (null != connectButton) {
        connectButton.onclick = onClickHandler
    }
}


// Sign Typed Data V4
const onClicksignTypedDataV4Handler = async () => {
    if (sb.ready()) {

        sb.setSignInProgress()

        try {
            const from = sb.getAccount();
            const msgParams = sb.getParams();
            DebugSpew("click - from: " + from)
            const sign = await ethereum.request({
                method: 'eth_signTypedData_v4',
                params: [from, msgParams],
            });
            sb.setSign(sign)
        } catch (err) {
            console.error(err);
            sb.setSignErr(`Error: ${err.message}`)
        }
    }
    else {
        DebugSpew("System not configured for signature!")
    }
}

// Assumes csignButton class is active, check for existance before calling.
function onClickSignSetup() {
    let signButton = sb.getsignButton()
    if (null != signButton) {
        signButton.onclick = onClicksignTypedDataV4Handler
    }
}


/*
This routine gets called when the DOM is finalized for the browser. It means that if the user
created a MetaMask button, we'll find it.

So ...
If the button exists on the page
  Initialize all html objects
  if MetaMask is installed
    Get Chain Id
    setup notification callbacks
    call eth_accounts to see what it linked.
    if there are linked accounts
      get the associated balance
  else
    set button to say 'connect'
    setup onClick handler.
else
  do nothing
*/
const initialize = async () => {
    // does the user want debug strings?
    if (dom.exists(turnondebugDiv, turnondebugStr)) {
        turnOnDebug = true;
        console.time()
    }

    // Testing code here...
    //let tWei = new CTestWei()
    //tWei.main();

    DebugSpew("initialize enter")

    if (cb.exists()) {

        if (cb.isMetaMaskInstalled()) {

            ethereum.autoRefreshOnNetworkChange = false
            getNetworkAndChainId()

            // These are change notifications from metamask
            ethereum.on('chainChanged', handleNewChain)
            ethereum.on('accountsChanged', handleNewAccounts)

            try {
                const newAccounts = await ethereum.request({
                    method: 'eth_accounts',
                })
                DebugSpew("back from eth_accounts on chain: " + cb.chainName())

                // success path

                cb.handleAccountChange(newAccounts)
                if (sb.exists()) {
                    sb.handleAccountChange(newAccounts)
                }

                // here is where we fetch the balance and block number!
                queryNetworkForData()

            } catch (err) {
                console.error(err)
            }

        }

        // always setup the clickability of the button
        onClickSetup()
        cb.updateState()
        if (sb.exists()) {
            onClickSignSetup()
        }
    }
    else {
        DebugSpew("initialize connectButton not present")
    }

    DebugSpew("initialize leave")
    if (turnOnDebug) {
        console.timeEnd()
    }
}

window.addEventListener('DOMContentLoaded', initialize)

// ---------------------------------------------------------------------------------------------------------------

/*


//  Sign Typed Data V4 Verification
signTypedDataV4Verify.onclick = async () => {
    const networkId = parseInt(networkDiv.innerHTML, 10);
    const chainId = parseInt(chainIdDiv.innerHTML, 16) || networkId;

    try {
        const from = accounts[0];
        const sign = signTypedDataV4Result.innerHTML;
        const recoveredAddr = recoverTypedSignatureV4({
            data: msgParams,
            sig: sign,
        });
        if (toChecksumAddress(recoveredAddr) === toChecksumAddress(from)) {
            console.log(`Successfully verified signer as ${recoveredAddr}`);
            signTypedDataV4VerifyResult.innerHTML = recoveredAddr;
        } else {
            console.log(
                `Failed to verify signer when comparing ${recoveredAddr} to ${from}`,
            );
        }
    } catch (err) {
        console.error(err);
        signTypedDataV4VerifyResult.innerHTML = `Error: ${err.message}`;
    }
};

*/



